#include "lc.h"
#include "psect.h"
#include "util.h"

int flatmodel = 1;	// Are we using a flat (no psects) model?
int lc;			// The (flat) location counter


void lcAdjustSize(void)
{
  /* lc is always relative to psect start. */
  if (curpsect->lc > curpsect->length) curpsect->length = curpsect->lc;
}


void lcSet(const int newlc)
{
  if (flatmodel) {
    lc = newlc;
  } else {
    (curpsect->lc) = newlc;
    lcAdjustSize();
  }
}


int lcGet(void)
{
  return (flatmodel ? lc : (curpsect->lc));
}


int lcInc(void)
{
  int oldlc = (flatmodel ? lc++ : (curpsect->lc)++);
  if (!flatmodel) lcAdjustSize();
  return oldlc;
}


int lcDec(void)
{
  int oldlc = (flatmodel ? lc-- : (curpsect->lc)--);
  if (!flatmodel) lcAdjustSize();
  return oldlc;
}


int lcGetAbs(void)
{
  if (flatmodel) return lc;
  if (curpsect->start < 0) {
    message(MSG_CRITICAL, "Attempt to get absolute lc value before psect start assigned for %s. Assembler error?", curpsect->pname);
    return curpsect->lc;
  }
  switch (curpsect->class) {
    case PS_CODE: return curpsect->start + curpsect->lc; break;
    case PS_EEDATA: return 0xf000 + curpsect->start + curpsect->lc; break;
    case PS_CONFIG: return 0x8000 + curpsect->start + curpsect->lc; break;
    case PS_COMMON: return curpsect->start + curpsect->lc; break;
    case PS_LINEAR: return curpsect->start + curpsect->lc; break;
    default:
      return curpsect->start + curpsect->lc;
  }
}

