#include <stdio.h>
#include <string.h>
#include "conf.h"



void srcStripeol(char *buf)
{
  int i = strlen(buf);

  if (i == 0) return;
  i--;
  do {
    if ((buf[i] == '\n') || (buf[i] == '\r')) {
      buf[i] = '\0';
      i--;
    } else break;
  } while (i);
}


void
srcEatComments(char *buf, char *comment)
{
  int i;
  char quoting = 0;

  if (comment) comment[0] = '\0';

  // Turn any cpp directives into comments, to allow preprocessing with cpp.
  //
  if (buf[0] == '#') {
    // Turn the '#' into ';' comment delimiter
    buf[0] = ';';
    // Copy it to 'comment'
    if (comment) strcpy(comment, buf);
    // Make 'buf' empty.
    buf[0] = '\0';
    return;
  }

  i = 0;
  while (buf[i]) {
    if (quoting && (buf[i] == quoting)) {
      quoting = 0;
    } else {
      if ((buf[i] == '\'') || (buf[i] == '"')) {
        quoting = buf[i];
      } else {
        if (buf[i] == ';') {
          // Found beginning of comment. Copy here to EOL to 'comment'
          if (comment) strcpy(comment, &(buf[i]));
          // Backspace to last non-whitespace
          do {
            i--;
          } while (i && (strchr(" \t", buf[i])));
          buf[i+1] = '\0';
          return;
        }
      }
    }
    i++;
  }
}


void srcBreakoutLine(const char *buf, char *l, char * opcode, char * operand)
{
  int i;
  char *tbuf;

  /* Make all sections blank in case we exit early */
  l[0] = '\0';
  opcode[0] = '\0';
  operand[0] = '\0';

  i = 0;

  /* Copy any label */
  tbuf = l;
  // while ((buf[i] != ' ') && (buf[i] != '\t') && (buf[i] != '\n')
  //     && (buf[i] != '\0') && (buf[i] != ';')) {
  while (buf[i] && (strchr(" \t;:", buf[i]) == NULL) && (i < (LABEL_LEN-1))) {
    *tbuf = buf[i];
    tbuf++;
    i++;
  }
  *tbuf = '\0';

  /* If we're at EOL, return with just a (possible) label */
  if ((buf[i] == '\0')) return;
  while (buf[i] && strchr(" \t:", buf[i]))
      i++;
  if (buf[i] == '\0') return;

  /* Copy any opcode */
  tbuf = opcode;
  while ((buf[i] != ' ') && (buf[i] != '\t')
      && (buf[i] != '\0') && (buf[i] != ';')) {
    *tbuf = buf[i];
    tbuf++;
    i++;
  }
  *tbuf = '\0';

  /* If we're at EOL, return with just a (possible) label and opcode */
  if (buf[i] == '\0') return;
  while (buf[i] && ((buf[i] == ' ') || (buf[i] == '\t'))) i++;
  if (buf[i] == '\0') return;

  /* Copy any operands */
  tbuf = operand;
  while (buf[i]) {
    *tbuf = buf[i];
    tbuf++;
    i++;
  }
  *tbuf = '\0';

}



int srcParseLine(char * buf, char *label, char *opcode, char *operand,
                    char *comment)
{
  if (comment) comment[0] = '\0';
  if ((buf[0] == '\0') || (buf[0] == '\n') || (buf[0] == '\r')) return 0;
  if (buf[0] == ';') {
    if (comment) strcpy(comment, buf);
    return 0;
  }
  srcStripeol(buf);
  srcEatComments(buf, comment);
  srcBreakoutLine(buf, label, opcode, operand);
  // fprintf(stderr, "INFO: label = '%s', opcode = '%s', operand = '%s'\n",
  //     label, opcode, operand);
  return 1;
}

